import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class FileImporter {

    private static final String DELIM = ",";
    private static final int MAX_SIZE = (1 << 20) * 10;

    /**
     * Imports a ratings array from a CSV file with the given filename.
     * @param filename the filname
     * @return a ratings array
     */
    public static byte[][] loadData(String filename) {
        int sizeX, sizeY;
        byte[][] out = null;
        BufferedReader br = null;
        try {
            br = new BufferedReader(new FileReader(filename));
            String line = br.readLine();
            if (line == null)
                throw new RuntimeException("Invalid file format");

            String[] dimensions = line.split(DELIM);

            if (dimensions.length != 2)
                throw new RuntimeException("Invalid file format");

            sizeX = Integer.valueOf(dimensions[0]);
            sizeY = Integer.valueOf(dimensions[1]);

            if (sizeX * sizeY > MAX_SIZE)
                throw new RuntimeException("Dimensions are too large");

            out = new byte[sizeX][sizeY];
            int x = 0;
            while ((line = br.readLine()) != null) {
                String[] values = line.split(DELIM);
                if (values.length != sizeY)
                    throw new RuntimeException("Illegal file format");

                for (int y = 0; y < values.length; y++) {
                    out[x][y] = (byte)(Integer.valueOf(values[y]) - 3);
                }
                x++;
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (br != null) {
                try {
                    br.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return out;
    }

    /**
     * Imports a titles array from a file with the given filename.
     * @param filename the filename
     * @return an array of movie titles
     */
    public static String[] loadTitles(String filename) {
        List < String > titles = new ArrayList < > ();
        BufferedReader br = null;
        try {
            br = new BufferedReader(new FileReader(filename));
            String line;
            while ((line = br.readLine()) != null) {
                titles.add(line);
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (br != null) {
                try {
                    br.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return titles.toArray(new String[0]);
    }
}