import java.util.Arrays;

public class FilmRecommender {

    // Constant defining the rating value if a user has not yet seen a particular
    // film
    static final int FILM_NOT_SEEN = -3;

    public static void main(String[] args) {
        // Example array of film titles
        String[] titles = {"Der boese Vogt", "Der Meisterschuetze",
                "Aepfel sind zum Essen da", "Ein Kind ist keine Zielscheibe"};

        // Example ratings array for 6 users and 4 films used in the task description.
        // -3 indicates that the user has not seen the corresponding film yet
        byte[][] ratings = {{-3, 2, -3, -1}, {-1, -3, 2, 1}, {2, 1, -2, -2},
                {1, 2, -3, -1}, {-2, -1, 1, 1}, {2, -3, -1, -3}};

        // Print results for the example
        System.out.println("----------  EXAMPLE  ----------");
        printRatings(ratings, titles);

        // Print results for the small dataset
        System.out.println("-------- SMALL DATASET --------");
        titles = FileImporter.loadTitles("data/SmallData1_s31415_titles.txt");
        ratings = FileImporter.loadData("data/SmallData1_s31415_data.txt");
        printRatings(ratings, titles);

        // Print results for the large dataset
        System.out.println("-------- LARGE DATASET --------");
        ratings = FileImporter.loadData("data/LargeData1_s31412_data.txt");
        titles = FileImporter.loadTitles("data/LargeData1_s31412_titles.txt");
        printRatings(ratings, titles);
    }

    /**
     * Computes film recommendations for each user and prints them to the console.
     *
     * @param ratings the ratings matrix
     * @param titles  an array that maps film id's to film titles.
     */
    public static void printRatings(byte[][] ratings, String[] titles) {
        for (int userId = 0; userId < ratings.length; userId++) {
            // Compute recommendation for a specific user
            int recommendation = computeRecommendation(ratings, userId);

            if (recommendation == -1) {
                System.out.println("User " + userId + " has already seen all films.");
            } else {
                System.out.println("We recommend \"" + titles[recommendation] + "\" for user " + userId + ".");
            }
        }
        System.out.println("-------------------------------\n");
    }

    /**
     * Computes the cosine similarity for to users based on their ratings for
     * commonly seen films. A ratings of -3 indicates that the user has not yet seen
     * the film. These films have to be omitted in the similarity calculation. If
     * there are no or only one commonly seen films, then a similarity of 0 will be
     * returned.
     *
     * @param ratings1 The ratings of the first user
     * @param ratings2 The rating of the second user
     * @return The cosine similarity of the two ratings vectors. Films with ratings
     * of -3 for either user are omitted in the calculation. A similarity of
     * 0 will be returned if there are no or only one commonly seen film or
     * if one of the ratings vectors is the zero vector.
     * @throws IllegalArgumentException if one or both of the ratings vectors are
     *                                  null or the ratings vectors are of unequal
     *                                  length
     */
    static double similarity(byte[] ratings1, byte[] ratings2) throws IllegalArgumentException {
        //TODO
        return -0.5;
    }

    /**
     * Computes the recommendation scores for unseen films for a given user based on his / her
     * similarity to the other users and their ratings. A user's ratings are given
     * in the rows of the matrix, a particular film's ratings are in the columns.
     *
     * @param ratings The rating matrix containing the ratings for all users.
     *                Ratings range from -2 to 2. A value of -3 indicates that the
     *                corresponding film has not been seen yet.
     * @param userId  The id of the user for whom the ratings for unseen films
     *                should be computed
     * @return The computed scores for each film in a double array for the given user at (userid).
     * The score for all seen films is zero.
     * The score for unseen films is computed based on the similarities to the other users and their ratings.
     * If there are no other users, the ratings vector from the given user is returned.
     * If all similarities for users who have seen a particular film are 0, then the score
     * for that film is computed as the (unweighted) average of the ratings.
     * If no other user has seen a particular film, the score for that film will be 0.
     * If the given user has seen all films, all scores will be 0.
     */
    static double[] computeRatings(byte[][] ratings, int userId) {
        // Create the array to store the computed ratings
        double[] computedRatings = new double[ratings[0].length];
        int nOtherUsers = ratings.length - 1;
        if (nOtherUsers == 0) {
            // In case there are no other users, return the ratings as is
            //TODO
        }
        // Calculate the similarity to all other users
        //TODO
        
        // Iterate over all films the user has not seen yet
        //TODO
        return null;
    }

    /**
     * Computes the recommendations for unseen films for user at index userId, and
     * returns an unseen film index with the maximum recommendation rating.
     * If multiple films have the same rating, the first observed maximum is returned.
     * The recommendations are based on user similarities given a user ratings
     * matrix. The recommendation is computed for the given user index (userId). A
     * user's ratings are given in the rows of the matrix, a particular film's
     * ratings are in the columns.
     *
     * @param ratings The rating matrix containing the ratings for all users.
     *                Ratings range from -2 to 2. A value of -3 indicates that the
     *                corresponding film has not been seen yet.
     * @param userId  The id of the user for whom the recommendation should be
     *                computed
     * @return The index of an unseen film for the user at index userId with the maximal recommendation score.
     * If multiple unseen films have the same score, the first observed maximum is returned.
     * If there are no other users the first unseen film in the array will be recommended.
     * If the user has seen (i.e. rated) all the films before, -1 will be returned.
     * If there are no other users, the first unseen film will be recommended.
     */
    static int computeRecommendation(byte[][] ratings, int userId) {
        double[] computedRatings = computeRatings(ratings, userId); 
        // If there are no other users, the ratings vector
        // will be the given user's ratings, which will lead to
        // the first unseen film to be recommended
        
        //TODO
        return -2;
    }
}